package cz.zcu.ups.arcaoftpd.connection;

import java.net.Socket;
import java.net.SocketAddress;

import cz.zcu.ups.arcaoftpd.client.ClientParameters;

/**
 * Trida obsluhuje aktivni pripojeni klienta
 * @author Arcao
 */
public class ActiveConnection extends Connection {
	private SocketAddress destination = null;

	/**
	 * Konstruktor tridy
	 * @param clientParameters		informace o pripojenem klientovi
	 * @param destination			adresa a port klienta, na kterem klient ceka na pripojeni serveru
	 */
	public ActiveConnection(ClientParameters clientParameters, SocketAddress destination) {
		super(clientParameters);
		this.destination = destination;
	}

	/**
	 * Nastavuje cil, na kterem ceka klient na pripojeni serveru
	 * @param destination			adresa a port, na kterem nasloucha klient
	 * @return						eference na (this) ActiveConnection
	 */
	public ActiveConnection setDestination(SocketAddress destination) {
		this.destination = destination;
		return this;
	}

	/**
	 * Vraci cil, na kterem ceka klient na pripojeni serveru
	 * @return						adresa a port, na kterem nasloucha klient
	 */
	public SocketAddress getDestination() {
		return destination;
	}

	/**
	 * Funkce vykonajici pripojeni ke klientovi a spusteni prirazene akce
	 */
	public void run() {
		try {
			setSocket(new Socket());
		} catch (Exception e) {
			try {
				getClientParameters().sendMessage(425, "Can't open data connection.");
			} catch(Exception ex) {
			}
			e.printStackTrace();
			return;
		}
		try {
			getAction().setClientParameters(getClientParameters());

			//spustime predakci (napr poslani nejakych kodu klientovi)
			getAction().preRun();
			// pripojime se ke klientovi, pokud se to nepovede do 60s, tak to vyhodi chybu
			getSocket().connect(destination, 60000);
			getAction().setSocket(getSocket());
			//spustime hlavni akci
			getAction().run();
		} catch (Exception e) {
			try {
				getClientParameters().sendMessage(426, "Connection closed; transfer aborted.");
			} catch (Exception ex) {
			}
			e.printStackTrace();
		}
	}

	/**
	 * Uvolni pouzite prostredky
	 */
	public void dispose() {
		try {
			if (getAction().getSocket() != null) {
				getAction().getSocket().close();
				getAction().setSocket(null);
			}
			setAction(null);
			setSocket(null);
		} catch (Exception e) {
		}

	}
}
