package cz.zcu.ups.arcaoftpd.dataaction;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.RandomAccessFile;
import java.util.Date;

import cz.zcu.ups.arcaoftpd.ArcaoFTPd;
import cz.zcu.ups.arcaoftpd.client.TransferMode;

/**
 * Trida zastresujici datovou akci, ktera prijima soubor od klienta
 * @author Arcao
 */
public class StoreDataAction extends DataAction {
	private static final int FILE_BUFFER_LEN = 4096;
	private File file;
	private boolean append = false;

	/**
	 * Konstriktor tridy s nastavenim cesty k souboru, ktery klient chce ulozit na serveru
	 * @param parameter		cesta k souboru
	 */
	public StoreDataAction(String parameter) {
		super(parameter);
	}

	/**
	 * Konstriktor tridy s nastavenim cesty k souboru, ktery klient chce ulozit na serveru a parametru urcujici, zda se bude ukladat na konec jiz existujiciho souboru
	 * @param parameter		cesta k souboru
	 * @param append		ulozit prijata data na konec souboru
	 */
	public StoreDataAction(String parameter, boolean append) {
		super(parameter);
		this.append = append;
	}

	/**
	 * Posleme klinetovi, ze jsme otevreli spojeni
	 */
	public void preRun() {
		try {
			file = getClientParameters().getUser().translatePathToAbsoluteFile(getParameter());
			getClientParameters().sendMessage(150, "Opening data connection for \"%s\".", file.getName());
			ArcaoFTPd.getLogger().write(("OD: " + new Date().toString() + "; " + getClientParameters().getSocket().getRemoteSocketAddress().toString() +"; Open data connection...\n").getBytes("utf-8"));
		} catch (Exception e) {
		}
	}

	/**
	 * Funkce starajici se o prijeti souboru od klienta
	 */
	public void run() {
		try {
			//je li spojeni v binarnim rezimu, prijmeme data binarne, jinak textove
			if (getClientParameters().getTransferMode() == TransferMode.Binary) {
				InputStream in = getSocket().getInputStream();
				RandomAccessFile out = new RandomAccessFile(file, "rw");
				if (append) {
					out.seek(file.length());
				} else {
					out.seek(getClientParameters().getFileSeekPosition());
				}
				getClientParameters().setFileSeekPosition(0);

				//Cteme data bufferovane po castech o velikosti FILE_BUFFER_LEN
		        byte[] buf = new byte[FILE_BUFFER_LEN];
		        int len;
		        while ((len = in.read(buf)) > 0) {
		            out.write(buf, 0, len);
		        }
		        in.close();
				out.close();
			} else {
				BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file)));
				BufferedReader in = new BufferedReader(new InputStreamReader(getSocket().getInputStream()));

				String line;
				//cteme vsechny radky ze socketu
				while((line = in.readLine()) != null) {
					out.write(line);
					out.newLine();
				}
				in.close();
				out.close();
			}

			getSocket().close();
			getClientParameters().sendMessage(226, "Transfer ok.");
		} catch(Exception e) {
			try {
				getClientParameters().sendMessage(426, "Connection closed; transfer aborted.");
			} catch (Exception ex) {
			}
			e.printStackTrace();
		}
		try {
			ArcaoFTPd.getLogger().write(("CD: " + new Date().toString() + "; " + getClientParameters().getSocket().getRemoteSocketAddress().toString() +": Close data connection...\n").getBytes("utf-8"));
		} catch (Exception e) {
		}
	}
}
