/**
 *
 */
package cz.zcu.ups.arcaoftpd.util;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.PrintStream;

/**
 * Trida starajici se o logovani do souboru
 * @author Arcao
 */
public class Logger {
	private OutputStream err = null, out = null;
	private PrintStream old_err = null;

	/**
	 * Konstruktor tridy
	 * @param accessLogFile		nazev souboru, kam se bude ukladat komunikace mezi klientem a serverem (null, pokud vypisovat na obrazovku)
	 * @param errorLogFile		nazev souboru, kam se budou ukladat chybove hlaseni (null, pokud vypisovat na obrazovku)
	 */
	public Logger(String accessLogFile, String errorLogFile) {
		try {
			if (accessLogFile != null) {
				out = new FileOutputStream(new File(accessLogFile), true);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		try {
			if (errorLogFile != null) {
				err = new FileOutputStream(new File(errorLogFile));
				old_err = System.err;
				System.setErr(new PrintStream(err));
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Zapise se pole bytu do souboru, kam se uklada komunikace mezi klientem a serverem
	 * @param b					pole bytu
	 */
	public synchronized void write(byte[] b) {
		try {
			if (out != null) {
				out.write(b);
				out.flush();
			} else {
				System.out.write(b);
				System.out.flush();
			}
		} catch(Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Zapise se radka do souboru, kam se uklada komunikace mezi klientem a serverem
	 * @param b					text radky
	 */
	public synchronized void println(String b) {
		try {
			if (out != null) {
				out.write((b + "\n").getBytes("utf-8"));
				out.flush();
			} else {
				System.out.write((b + "\n").getBytes("utf-8"));
				System.out.flush();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Zapise se pole bytu do souboru, kam se ukladaji chybova hlaseni
	 * @param b					pole bytu
	 */
	public synchronized void writeErr(byte[] b) {
		try {
			if (err != null) {
				err.write(b);
				err.flush();
			} else {
				System.err.write(b);
				System.err.flush();
			}
		} catch(Exception e) {
		}
	}

	/**
	 * Zapise se radka do souboru, kam se ukladaji chybova hlaseni
	 * @param b					text radky
	 */
	public synchronized void printlnErr(String b) {
		try {
			if (err != null) {
				err.write((b + "\n").getBytes("utf-8"));
				err.flush();
			} else {
				System.err.write((b + "\n").getBytes("utf-8"));
				System.err.flush();
			}
		} catch(Exception e) {
		}
	}


	/**
	 * Destruktor objektu, ktery vrati vsechna nastaveni do puvodni podoby (uklidime po sobe)
	 */
	@Override
	protected void finalize() throws Throwable {
		if (out != null) out.close();
		if (err != null) err.close();
		if (old_err != null) System.setErr(old_err);
		super.finalize();
	}
}
