package cz.zcu.ups.arcaoftpd.connection;

import java.net.Socket;

import cz.zcu.ups.arcaoftpd.client.ClientParameters;
import cz.zcu.ups.arcaoftpd.dataaction.DataAction;

/**
 * Trida definuje zakladni vlastnosti pro spojeni
 * @author Arcao
 */
public abstract class Connection extends Thread {
	private DataAction action;
	private ClientParameters clientParameters;
	private Socket socket = null;

	/**
	 * Kontruktor tridy s urcenim parametru klienta
	 * @param clientParameters			parametry klienta
	 */
	public Connection(ClientParameters clientParameters) {
		this.clientParameters = clientParameters;
	}

	/**
	 * Provede navazani spojeni (spusti {@linkplain #run()} jako nove vlakno)
	 * @return						reference na (this) Connection
	 */
	public Connection connect() {
		start();
		return this;
	}

	/**
	 * Vraci akci, ktera se vykonna po spojeni
	 * @return t					akce
	 */
	public DataAction getAction() {
		return action;
	}

	/**
	 * Nastavi akci, ktera se vykonna po spojeni
	 * @param action				akce
	 * @return						reference na (this) Connection
	 */
	public Connection setAction(DataAction action) {
		this.action = action;
		return this;
	}

	/**
	 * Vraci parametry klienta
	 * @return 						parametry klienta
	 */
	public ClientParameters getClientParameters() {
		return clientParameters;
	}

	/**
	 * Nastavuje parametry klienta
	 * @param clientParameters		parametry klienta
	 * @return						referenci na (this) Connection
	 */
	public Connection setClientParameters(ClientParameters clientParameters) {
		this.clientParameters = clientParameters;
		return this;
	}

	/**
	 * Vraci datovy socket
	 * @return						datovy socket
	 */
	public Socket getSocket() {
		return socket;
	}

	/**
	 * Nastaveni datoveho socketu
	 * @param socket 				datovy socket
	 * @return						referenci na (this) Connection
	 */
	public Connection setSocket(Socket socket) {
		this.socket = socket;
		return this;
	}

	/**
	 * Funkce vykonajici pripojeni ke klientovi a spusteni prirazene akce
	 */
	public abstract void run();

	/**
	 * Uvolni pouzite prostredky
	 */
	public abstract void dispose();
}
