/**
 *
 */
package cz.zcu.ups.arcaoftpd.connection;

import java.net.InetSocketAddress;
import java.net.ServerSocket;

import cz.zcu.ups.arcaoftpd.client.ClientParameters;

/**
 * Trida obsluhuje pasivni pripojeni klienta
 * @author Arcao
 */
public class PassiveConnection extends Connection {
	private ServerSocket ss;

	/**
	 * Konstruktor tridy
	 * @param clientParameters		informace o pripojenem klientovi
	 */
	public PassiveConnection(ClientParameters clientParameters) {
		super(clientParameters);

		try {
			//vytvorime server socket
			ss = new ServerSocket();
			//nastavime ip adresu naseho ftp serveru, na kterou je pripojen klient a vyberem volny port
			ss.bind(new InetSocketAddress(getClientParameters().getSocket().getLocalAddress(),0));

			// ip adresa
			int h1 = ss.getInetAddress().getAddress()[0] & 0xff;
			int h2 = ss.getInetAddress().getAddress()[1] & 0xff;
			int h3 = ss.getInetAddress().getAddress()[2] & 0xff;
			int h4 = ss.getInetAddress().getAddress()[3] & 0xff;


			// high a low byte portu
			int p1 = ss.getLocalPort() / 256;
			int p2 = ss.getLocalPort() % 256;

			// vratime nastaveni ip adresy a portu
			getClientParameters().sendMessage(227, "Entering Passive Mode (%d,%d,%d,%d,%d,%d)", h1, h2, h3, h4, p1, p2);

		} catch(Exception e) {
			try {
				getClientParameters().sendMessage(426, "Connection closed; transfer aborted.");
			} catch (Exception ex) {
			}
			e.printStackTrace();
		}
	}

	/**
	 * Funkce pocka, nez se pripoji klient, pak spusti prirazenou akci
	 */
	public void run() {
		try {
			if (ss == null) {
				getClientParameters().sendMessage(425, "Can't open data connection.");
				return;
			}
			getAction().setClientParameters(getClientParameters());
			getAction().preRun();

			// pokud do 60s se client nepripoji, vracime chybu
			ss.setSoTimeout(60000);
			setSocket(ss.accept());

			getAction().setSocket(getSocket());
			getAction().run();
		} catch (Exception e) {
			try {
				getClientParameters().sendMessage(426, "Connection closed; transfer aborted.");
			} catch (Exception ex) {
			}
			e.printStackTrace();
		}

	}

	/**
	 * Uvolni pouzite prostredky
	 */
	@Override
	public void dispose() {
		try {
			if (getAction().getSocket() != null) {
				getAction().getSocket().close();
				getAction().setSocket(null);
			}
			setAction(null);
			setSocket(null);
			ss.close();
			ss = null;
		} catch(Exception e) {
		}
	}
}
