package cz.zcu.ups.arcaoftpd.dataaction;

import java.net.Socket;

import cz.zcu.ups.arcaoftpd.client.ClientParameters;

/**
 * Abstractni trida zastresujici datovou akci
 * @author Arcao
 *
 */
public abstract class DataAction {
	private Socket socket;
	private ClientParameters clientParameters;
	private String parameter = "";

	/**
	 * Konstruktor tridy
	 */
	public DataAction() {
	}

	/**
	 * Konstruktor tridy s urcenim parametru k akci
	 * @param parameter				parametr k akci
	 */
	public DataAction(String parameter) {
		this.parameter =  parameter;
	}

	/**
	 * Nastaveni parametru klienta
	 * @param clientParameters		parametry klienta
	 * @return						reference ne (this) DataAction
	 */
	public DataAction setClientParameters(ClientParameters clientParameters) {
		this.clientParameters = clientParameters;
		return this;
	}

	/**
	 * Vraci parametry klienta
	 * @return						pareametry klienta
	 */
	public ClientParameters getClientParameters() {
		return clientParameters;
	}

	/**
	 * Nastavuje socket datoveho spojeni
	 * @param socket				socket datoveho spojeni
	 * @return						referenci na (this) DataAction
	 */
	public DataAction setSocket(Socket socket) {
		this.socket = socket;
		return this;
	}

	/**
	 * Vraci socket datoveho spojeni
	 * @return						socket datoveho spojeni
	 */
	public Socket getSocket() {
		return socket;
	}

	/**
	 * Nastavuje parametr akce
	 * @param parameter				parametr akce
	 * @return						reference na (this) DataAction
	 */
	public DataAction setParameter(String parameter) {
		this.parameter = parameter;
		return this;
	}

	/**
	 * Vraci parametr akce
	 * @return						parametr akce
	 */
	public String getParameter() {
		return parameter;
	}

	/**
	 * Pred akce spojeni, vykonna se pred navazanim spojeni, atp.
	 */
	public void preRun() {
	}

	/**
	 * Spusteni hlavni akce
	 */
	public abstract void run();
}
