package cz.zcu.ups.arcaoftpd.users;

import java.io.File;
import java.util.Arrays;

/**
 * Trida zastresujici uzivatele a prava na soubory
 * @author Arcao
 */
public class User {
	private String username;
	private String password;
	private boolean passwordReq;
	private Map[] maps;

	/**
	 * Konstruktor tridy nastavuji potrebne hodnoty
	 * @param username				uzivatelske jmeno
	 * @param password				heslo
	 * @param passwordReq			je heslo pozadovano
	 * @param maps					pole namapovanych adresaru
	 */
	public User(String username, String password, boolean passwordReq, Map[] maps) {
		this.username = username;
		this.password = password;
		this.passwordReq = passwordReq;
		this.maps = maps;
		Arrays.sort(this.maps);

		//System.out.println(username);
		//for (Map map : this.maps) {
		//	System.out.println(": " + map.getDestination());
		//}
		//System.out.println(translatePathToAbsolute("d:\\b.txt"));
		//System.out.println(exists("/"));
	}

	/**
	 * Vraci namapovane adresare
	 * @return 			pole namapovanych adresaru
	 */
	public Map[] getMaps() {
		return maps;
	}

	/**
	 * Nastavuje namapovane adresare
	 * @param maps		pole namapovanych adresaru
	 * @return			referenci na (this) User
	 */
	public User setMaps(Map[] maps) {
		this.maps = maps;
		return this;
	}

	/**
	 * Vraci heslo uzivatele
	 * @return			heslo uzivatele
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * Nastavuje heslo uzivatele
	 * @param password	heslo uzivatele
	 * @return			referenci na (this) User
	 */
	public User setPassword(String password) {
		this.password = password;
		return this;
	}

	/**
	 * Vraci, zda je pozadovano heslo
	 * @return			je pozadovano heslo?
	 */
	public boolean isPasswordReq() {
		return passwordReq;
	}

	/**
	 * Nastavuje, zda je pozadovano heslo
	 * @param passwordReq		je pozadovano heslo?
	 * @return					reference na (this) User
	 */
	public User setPasswordReq(boolean passwordReq) {
		this.passwordReq = passwordReq;
		return this;
	}

	/**
	 * Vraci uzivatelske jmeno
	 * @return			uzivatelske jmeno
	 */
	public String getUsername() {
		return username;
	}

	/**
	 * Nastavuje uzivatelske jmeno
	 * @param username	uzivatelske jmeno
	 * @return			referenci na (this) User
	 */
	public User setUsername(String username) {
		this.username = username;
		return this;
	}

	/**
	 * Ma soubor ci adresar prava pro cteni
	 * @param path		relativni cesta k souboru nebo adresari
	 * @return			pravo pro cteni?
	 */
	public boolean isPathReadable(String path) {
		for (Map map : maps)
			if (path.length() >= map.getDestination().length() && path.substring(0, map.getDestination().length()).equals(map.getDestination()))
				if (isFile(path)) {
					return (map.isReadable() && new File(translatePathToAbsolute(path, map)).canRead());
				} else {
					return map.isReadable();
				}
		return false;
	}

	/**
	 * Ma soubor ci adresar prava pro zapis
	 * @param path		relativni cesta k souboru nebo adresari
	 * @return			pravo pro zapis?
	 */
	public boolean isPathWriteable(String path) {
		for (Map map : maps)
			if (path.length() >= map.getDestination().length() && path.substring(0, map.getDestination().length()).equals(map.getDestination()))
				if (isFile(path)) {
					return (map.isWriteable() && new File(translatePathToAbsolute(path, map)).canWrite());
				} else {
					return map.isWriteable();
				}
		return false;
	}

	/**
	 * Ma adresar prava pro vylistovani
	 * @param path		relativni cesta k adresari
	 * @return			pravo pro vylistovani adresare?
	 */
	public boolean isPathListable(String path) {
		for (Map map : maps)
			if (path.length() >= map.getDestination().length() && path.substring(0, map.getDestination().length()).equals(map.getDestination()))
				return map.isListable() && isDirecory(path);
		return false;
	}

	/**
	 * Ma soubor prava pro mazani
	 * @param path		relativni cesta k souboru
	 * @return			pravo pro mazani?
	 */
	public boolean isPathFileDeleteable(String path) {
		for (Map map : maps)
			if (path.length() >= map.getDestination().length() && path.substring(0, map.getDestination().length()).equals(map.getDestination()))
				return (map.isFilesDeleteable() && new File(translatePathToAbsolute(path, map)).canWrite());
		return false;
	}

	/**
	 * Ma adresar prava pro mazani
	 * @param path		relativni cesta k adresari
	 * @return			pravo pro mazani?
	 */
	public boolean isPathDirectoryDeleteable(String path) {
		for (Map map : maps)
			if (path.length() >= map.getDestination().length() && path.substring(0, map.getDestination().length()).equals(map.getDestination()))
				return map.isDirectoryDeleteable();
		return false;
	}

	/**
	 * Ma soubor prava pro append
	 * @param path		relativni cesta k souboru
	 * @return			pravo pro append?
	 */
	public boolean isPathFileAppendable(String path) {
		for (Map map : maps)
			if (path.length() >= map.getDestination().length() && path.substring(0, map.getDestination().length()).equals(map.getDestination()))
				return (map.isAppendable() && new File(translatePathToAbsolute(path, map)).canWrite());
		return false;
	}

	/**
	 * Ma cesta pravo pro zalozeni adresare
	 * @param path		relativni cesta k adresari
	 * @return			pravo pro zalozeni adresare?
	 */
	public boolean isPathDirectoryMakeable(String path) {
		for (Map map : maps)
			if (path.length() >= map.getDestination().length() && path.substring(0, map.getDestination().length()).equals(map.getDestination()))
				return (map.isDirectoryMakeable());
		return false;
	}

	/**
	 * Je zadana cesta adresar
	 * @param path		cesta
	 * @return			je to adresar?
	 */
	public boolean isDirecory(String path) {
		return new File(translatePathToAbsolute(path)).isDirectory();
	}

	/**
	 * Je zadana cesta soubor
	 * @param path		cesta
	 * @return			je to soubor?
	 */
	public boolean isFile(String path) {
		return new File(translatePathToAbsolute(path)).isFile();
	}

	/**
	 * Existuje zadana cesta
	 * @param path		cesta
	 * @return			existuje?
	 */
	public boolean exists(String path) {
		return new File(translatePathToAbsolute(path)).exists();
	}

	/**
	 * Prevede relativni cestu na absolutni fyzickou
	 * @param path		cesta
	 * @return			absolutni fyzicka cesta
	 */
	public String translatePathToAbsolute(String path) {
		for (Map map : maps)
			if (path.length() >= map.getDestination().length() && path.substring(0, map.getDestination().length()).equals(map.getDestination()))
				return new File(map.getSource(), path.substring(map.getDestination().length())).getAbsolutePath();
		return null;
	}

	/**
	 * Prevede relativni cestu na absolutni fyzickou
	 * @param path		cesta
	 * @param map		mapovani, ktere se pouzije pro prevod na absolutni fyzickou cestu
	 * @return			absolutni fyzicka cesta
	 */
	public String translatePathToAbsolute(String path, Map map) {
		return new File(map.getSource(), path.substring(map.getDestination().length())).getAbsolutePath();
	}

	/**
	 * Prevede relativni cestu na absolutni fyzickou a vrati ji pres objekt File
	 * @param path		cesta
	 * @return 			absolutni fyzicka cesta jako objekt File
	 */
	public File translatePathToAbsoluteFile(String path) {
		for (Map map : maps)
			if (path.length() >= map.getDestination().length() && path.substring(0, map.getDestination().length()).equals(map.getDestination()))
				return new File(map.getSource(), path.substring(map.getDestination().length()));
		return null;
	}

	/**
	 * Prevede relativni cestu na absolutni fyzickou a vrati ji pres objekt File
	 * @param path		cesta
	 * @param map		mapovani, ktere se pouzije pro prevod na absolutni fyzickou cestu
	 * @return 			absolutni fyzicka cesta jako objekt File
	 */
	public File translatePathToAbsoluteFile(String path, Map map) {
		return new File(map.getSource(), path.substring(map.getDestination().length()));
	}
}
